==> insighttoolkit_3.14.0.orig.tar.gz: insighttoolkit-3.14.0/Testing/Code/BasicFilters/itkStatisticsImageFilterTest.cxx <==
/*=========================================================================

  Program:   Insight Segmentation & Registration Toolkit
  Module:    $RCSfile: itkStatisticsImageFilterTest.cxx,v $
  Language:  C++
  Date:      $Date: 2009-04-06 00:19:18 $
  Version:   $Revision: 1.12 $

  Copyright (c) Insight Software Consortium. All rights reserved.
  See ITKCopyright.txt or http://www.itk.org/HTML/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#if defined(_MSC_VER)
#pragma warning ( disable : 4786 )
#endif

#include <iostream>

#include "itkImage.h"
#include "itkImageRegionIterator.h"
#include "itkMersenneTwisterRandomVariateGenerator.h"

#include "itkStatisticsImageFilter.h"
#include "itkRandomImageSource.h"
#include "itkFilterWatcher.h"
#include "vnl/vnl_math.h"

int itkStatisticsImageFilterTest(int, char* [] )
{
  std::cout << "itkStatisticsImageFilterTest Start" << std::endl;

  int status = 0;
  typedef itk::Image<int,3> FloatImage;

  FloatImage::Pointer    image  = FloatImage::New();
  FloatImage::RegionType region;
  FloatImage::SizeType   size; size.Fill(64);
  FloatImage::IndexType  index; index.Fill(0);

  region.SetIndex (index);
  region.SetSize (size);

  // first try a constant image
  float fillValue = -100.0;
  image->SetRegions( region );
  image->Allocate();
  image->FillBuffer( static_cast< FloatImage::PixelType >( fillValue ) );

  float sum = fillValue * static_cast<float>( region.GetNumberOfPixels() );

  typedef itk::StatisticsImageFilter<FloatImage> FilterType;
  FilterType::Pointer filter = FilterType::New();

  FilterWatcher filterWatch(filter);
  
  filter->SetInput (image);
  filter->UpdateLargestPossibleRegion();

  if (filter->GetMinimum() != fillValue)
    {
    std::cerr << "GetMinimum failed! Got " << filter->GetMinimum() << " but expected " << fillValue << std::endl;
    status++;
    }
  if (filter->GetMaximum() != fillValue)
    {
    std::cerr << "GetMaximum failed! Got " << filter->GetMaximum() << " but expected " << fillValue << std::endl;
    status++;
    }
  if (filter->GetSum() != sum)
    {
    std::cerr << "GetSum failed! Got " << filter->GetSum() << " but expected " << sum << std::endl;
    status++;
    }
  if (filter->GetMean() != fillValue)
    {
    std::cerr << "GetMean failed! Got " << filter->GetMean() << " but expected " << fillValue << std::endl;
    status++;
    }
  if (filter->GetVariance() != 0.0)
    {
    std::cerr << "GetVariance failed! Got " << filter->GetVariance() << " but expected " << 0.0 << std::endl;
    status++;
    }


  // Now generate a real image

  typedef itk::RandomImageSource<FloatImage> SourceType;
  SourceType::Pointer source = SourceType::New();
  unsigned long randomSize[3] = {17, 8, 20};

  source->SetSize(randomSize);
  float minValue = -100.0;
  float maxValue = 1000.0;

  source->SetMin( static_cast< FloatImage::PixelType >( minValue ) );
  source->SetMax( static_cast< FloatImage::PixelType >( maxValue ) );

  filter->SetInput(source->GetOutput());
  filter->UpdateLargestPossibleRegion();
  
  double expectedSigma = vcl_sqrt((maxValue-minValue)*(maxValue-minValue)/12.0);
  double epsilon = (maxValue - minValue) * .001;

  if (vnl_math_abs(filter->GetSigma() - expectedSigma) > epsilon)
    {
    std::cerr << "GetSigma failed! Got " << filter->GetSigma() << " but expected " << expectedSigma << std::endl;
    }

  // Now generate an image with a known mean and variance
  itk::Statistics::MersenneTwisterRandomVariateGenerator::Pointer rvgen = itk::Statistics::MersenneTwisterRandomVariateGenerator::New();
  double knownMean = 12.0;
  double knownVariance = 10.0;

  typedef itk::Image<double,3> DoubleImage;
  DoubleImage::Pointer dImage = DoubleImage::New();
  DoubleImage::SizeType dsize;
  DoubleImage::IndexType dindex;
  DoubleImage::RegionType dregion;
  dsize.Fill(50);
  dindex.Fill(0);
  dregion.SetSize(dsize);
  dregion.SetIndex(dindex);
  dImage->SetRegions(dregion);
  dImage->Allocate();
  itk::ImageRegionIterator<DoubleImage> it(dImage, dregion);
  while (!it.IsAtEnd())
    {
    it.Set(rvgen->GetNormalVariate(knownMean, knownVariance));
    ++it;
    }
  typedef itk::StatisticsImageFilter<DoubleImage> DFilterType;
  DFilterType::Pointer dfilter = DFilterType::New();
  dfilter->SetInput(dImage);
  dfilter->UpdateLargestPossibleRegion();
  double testMean = dfilter->GetMean();
  double testVariance = dfilter->GetVariance();
  double diff = vnl_math_abs(testMean - knownMean);
  if ((diff != 0.0 && knownMean != 0.0) &&
      diff / vnl_math_abs(knownMean) > .01)
    {
    std::cout << "Expected mean is " << knownMean << ", computed mean is " << testMean << std::endl;
    return EXIT_FAILURE;
    }
  std::cout << "Expected mean is " << knownMean << ", computed mean is " << testMean << std::endl;
  diff = vnl_math_abs(testVariance - knownVariance);
  if ((diff != 0.0 && knownVariance != 0.0) &&
      diff / vnl_math_abs(knownVariance) > .1)
    {
    std::cout << "Expected variance is " << knownVariance << ", computed variance is " << testVariance << std::endl;
    return EXIT_FAILURE;
    }
  std::cout << "Expected variance is " << knownVariance << ", computed variance is " << testVariance << std::endl;
  return status;
}

==> insighttoolkit_3.18.0.orig.tar.gz: insighttoolkit-3.18.0/Testing/Code/BasicFilters/itkStatisticsImageFilterTest.cxx <==
/*=========================================================================

  Program:   Insight Segmentation & Registration Toolkit
  Module:    $RCSfile: itkStatisticsImageFilterTest.cxx,v $
  Language:  C++
  Date:      $Date: 2009-07-12 10:53:00 $
  Version:   $Revision: 1.13 $

  Copyright (c) Insight Software Consortium. All rights reserved.
  See ITKCopyright.txt or http://www.itk.org/HTML/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#if defined(_MSC_VER)
#pragma warning ( disable : 4786 )
#endif

#include <iostream>

#include "itkImage.h"
#include "itkImageRegionIterator.h"
#include "itkMersenneTwisterRandomVariateGenerator.h"

#include "itkStatisticsImageFilter.h"
#include "itkRandomImageSource.h"
#include "itkFilterWatcher.h"
#include "vnl/vnl_math.h"

int itkStatisticsImageFilterTest(int, char* [] )
{
  std::cout << "itkStatisticsImageFilterTest Start" << std::endl;

  int status = 0;
  typedef itk::Image<int,3> FloatImage;

  FloatImage::Pointer    image  = FloatImage::New();
  FloatImage::RegionType region;
  FloatImage::SizeType   size; size.Fill(64);
  FloatImage::IndexType  index; index.Fill(0);

  region.SetIndex (index);
  region.SetSize (size);

  // first try a constant image
  float fillValue = -100.0;
  image->SetRegions( region );
  image->Allocate();
  image->FillBuffer( static_cast< FloatImage::PixelType >( fillValue ) );

  float sum = fillValue * static_cast<float>( region.GetNumberOfPixels() );

  typedef itk::StatisticsImageFilter<FloatImage> FilterType;
  FilterType::Pointer filter = FilterType::New();

  FilterWatcher filterWatch(filter);
  
  filter->SetInput (image);
  filter->UpdateLargestPossibleRegion();

  if (filter->GetMinimum() != fillValue)
    {
    std::cerr << "GetMinimum failed! Got " << filter->GetMinimum() << " but expected " << fillValue << std::endl;
    status++;
    }
  if (filter->GetMaximum() != fillValue)
    {
    std::cerr << "GetMaximum failed! Got " << filter->GetMaximum() << " but expected " << fillValue << std::endl;
    status++;
    }
  if (filter->GetSum() != sum)
    {
    std::cerr << "GetSum failed! Got " << filter->GetSum() << " but expected " << sum << std::endl;
    status++;
    }
  if (filter->GetMean() != fillValue)
    {
    std::cerr << "GetMean failed! Got " << filter->GetMean() << " but expected " << fillValue << std::endl;
    status++;
    }
  if (filter->GetVariance() != 0.0)
    {
    std::cerr << "GetVariance failed! Got " << filter->GetVariance() << " but expected " << 0.0 << std::endl;
    status++;
    }


  // Now generate a real image

  typedef itk::RandomImageSource<FloatImage> SourceType;
  SourceType::Pointer source = SourceType::New();

  FloatImage::SizeValueType randomSize[3] = {17, 8, 20};

  source->SetSize(randomSize);
  float minValue = -100.0;
  float maxValue = 1000.0;

  source->SetMin( static_cast< FloatImage::PixelType >( minValue ) );
  source->SetMax( static_cast< FloatImage::PixelType >( maxValue ) );

  filter->SetInput(source->GetOutput());
  filter->UpdateLargestPossibleRegion();
  
  double expectedSigma = vcl_sqrt((maxValue-minValue)*(maxValue-minValue)/12.0);
  double epsilon = (maxValue - minValue) * .001;

  if (vnl_math_abs(filter->GetSigma() - expectedSigma) > epsilon)
    {
    std::cerr << "GetSigma failed! Got " << filter->GetSigma() << " but expected " << expectedSigma << std::endl;
    }

  // Now generate an image with a known mean and variance
  itk::Statistics::MersenneTwisterRandomVariateGenerator::Pointer rvgen = itk::Statistics::MersenneTwisterRandomVariateGenerator::New();
  double knownMean = 12.0;
  double knownVariance = 10.0;

  typedef itk::Image<double,3> DoubleImage;
  DoubleImage::Pointer dImage = DoubleImage::New();
  DoubleImage::SizeType dsize;
  DoubleImage::IndexType dindex;
  DoubleImage::RegionType dregion;
  dsize.Fill(50);
  dindex.Fill(0);
  dregion.SetSize(dsize);
  dregion.SetIndex(dindex);
  dImage->SetRegions(dregion);
  dImage->Allocate();
  itk::ImageRegionIterator<DoubleImage> it(dImage, dregion);
  while (!it.IsAtEnd())
    {
    it.Set(rvgen->GetNormalVariate(knownMean, knownVariance));
    ++it;
    }
  typedef itk::StatisticsImageFilter<DoubleImage> DFilterType;
  DFilterType::Pointer dfilter = DFilterType::New();
  dfilter->SetInput(dImage);
  dfilter->UpdateLargestPossibleRegion();
  double testMean = dfilter->GetMean();
  double testVariance = dfilter->GetVariance();
  double diff = vnl_math_abs(testMean - knownMean);
  if ((diff != 0.0 && knownMean != 0.0) &&
      diff / vnl_math_abs(knownMean) > .01)
    {
    std::cout << "Expected mean is " << knownMean << ", computed mean is " << testMean << std::endl;
    return EXIT_FAILURE;
    }
  std::cout << "Expected mean is " << knownMean << ", computed mean is " << testMean << std::endl;
  diff = vnl_math_abs(testVariance - knownVariance);
  if ((diff != 0.0 && knownVariance != 0.0) &&
      diff / vnl_math_abs(knownVariance) > .1)
    {
    std::cout << "Expected variance is " << knownVariance << ", computed variance is " << testVariance << std::endl;
    return EXIT_FAILURE;
    }
  std::cout << "Expected variance is " << knownVariance << ", computed variance is " << testVariance << std::endl;
  return status;
}
